'use strict';

exports.__esModule = true;
exports.Motion = undefined;

var _react = require('react');

var _TransitionMotion = require('react-motion/lib/TransitionMotion');

var _TransitionMotion2 = _interopRequireDefault(_TransitionMotion);

var _spring = require('react-motion/lib/spring');

var _spring2 = _interopRequireDefault(_spring);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var Motion = exports.Motion = function (_PureComponent) {
  _inherits(Motion, _PureComponent);

  function Motion(props) {
    _classCallCheck(this, Motion);

    var _this = _possibleConstructorReturn(this, _PureComponent.call(this, props));

    _this.renderCurrentStyles = _this.renderCurrentStyles.bind(_this);
    _this.getKey = _this.getKey.bind(_this);
    _this.getDefaultStyles = _this.getDefaultStyles.bind(_this);
    _this.getStyles = _this.getStyles.bind(_this);
    _this.willLeave = _this.willLeave.bind(_this);
    _this.willEnter = _this.willEnter.bind(_this);
    return _this;
  }

  Motion.prototype.render = function render() {
    var _props = this.props,
        data = _props.data,
        onRemount = _props.onRemount,
        onUnmount = _props.onUnmount,
        onComponentMount = _props.onComponentMount;


    return (0, _react.createElement)(_TransitionMotion2.default, {
      defaultStyles: data.length && onComponentMount ? this.getDefaultStyles() : undefined,
      styles: this.getStyles(),
      willEnter: onRemount && this.willEnter,
      willLeave: onUnmount && this.willLeave,
      children: this.renderCurrentStyles
    });
  };

  Motion.prototype.renderCurrentStyles = function renderCurrentStyles(currentStyles) {
    var _props2 = this.props,
        component = _props2.component,
        render = _props2.render;

    var children = void 0;
    if (Array.isArray(render)) {
      // If render is an array, children becomes and array of arrays.
      // This is useful if you want to create layers of animation
      children = new Array(render.length);
      for (var i = 0; i < render.length; ++i) {
        children[i] = new Array(currentStyles.length);
        for (var j = 0; j < currentStyles.length; ++j) {
          var _currentStyles$j = currentStyles[j],
              key = _currentStyles$j.key,
              data = _currentStyles$j.data,
              style = _currentStyles$j.style;

          children[i][j] = render[i](key, data, style, j, i);
        }
      }
    } else {
      children = new Array(currentStyles.length);
      for (var _j = 0; _j < currentStyles.length; ++_j) {
        var _currentStyles$_j = currentStyles[_j],
            key = _currentStyles$_j.key,
            data = _currentStyles$_j.data,
            style = _currentStyles$_j.style;

        children[_j] = render(key, data, style, _j, 0);
      }
    }

    return (0, _react.cloneElement)(component, { children: children });
  };

  Motion.prototype.getKey = function getKey(data, i) {
    var getKey = this.props.getKey;

    if (typeof getKey === 'function') {
      return getKey(data, i);
    }

    return getKey;
  };

  Motion.prototype.getDefaultStyles = function getDefaultStyles() {
    var _props3 = this.props,
        getKey = _props3.getKey,
        onComponentMount = _props3.onComponentMount,
        data = _props3.data;


    var mappedData = new Array(data.length);
    for (var i = 0; i < data.length; ++i) {
      mappedData[i] = {
        key: getKey(data[i], i),
        data: data[i],
        style: onComponentMount(data[i], i)
      };
    }

    return mappedData;
  };

  Motion.prototype.getStyles = function getStyles() {
    var _props4 = this.props,
        getKey = _props4.getKey,
        onRender = _props4.onRender,
        data = _props4.data;


    var mappedData = new Array(data.length);
    for (var i = 0; i < data.length; ++i) {
      mappedData[i] = {
        key: getKey(data[i], i),
        data: data[i],
        style: onRender(data[i], i, _spring2.default)
      };
    }

    return mappedData;
  };

  Motion.prototype.willEnter = function willEnter(config) {
    return this.props.onRemount(config);
  };

  Motion.prototype.willLeave = function willLeave(config) {
    return this.props.onUnmount(config, _spring2.default);
  };

  return Motion;
}(_react.PureComponent);

Motion.defaultProps = {
  component: (0, _react.createElement)('div'),
  data: []
};

process.env.NODE_ENV !== "production" ? Motion.propTypes = {
  // wrapper element ex: <div/>
  component: _react.PropTypes.element,

  // array of data
  data: _react.PropTypes.array,

  // (key, data, style) => ReactElement
  // called on render for each item in `data` with the key from `getKey`, `data` from `data[index]`, and
  // `style`, the result of: `onComponentMount`, `onRender`, `onRemount`, or `onUnmount`
  //
  // If an array is provided it will render the resulting elements in order.
  // This is useful for creating animated layers.
  render: _react.PropTypes.oneOfType([_react.PropTypes.func, _react.PropTypes.arrayOf(_react.PropTypes.func)]),

  // (data, i) => string
  // help identify which items have been changed, added, or are removed
  // https://facebook.github.io/react/docs/lists-and-keys.html
  getKey: _react.PropTypes.func,

  // (data, i) => Style object
  // data === props.data[i]
  // called when `component` mounts
  // do not wrap values in springs
  onComponentMount: _react.PropTypes.func,

  // (data, i, spring) => Style object
  // data === props.data[i]
  // called on every render
  // ok to wrap values in springs
  onRender: _react.PropTypes.func,

  // ({ data }, i) => Style object
  // data === props.data[i]
  // do not wrap values in springs
  onRemount: _react.PropTypes.func,

  // ({ data }, i, spring) => Style object
  // data === props.data[i]
  // ok to wrap values in springs
  onUnmount: _react.PropTypes.func
} : void 0;